;;; cps-tail.ss
;;; Copyright (c) 2005, R. Kent Dybvig, Simon L. Peyton Jones, and Amr Sabry

;;; CPS-based tail-recursive implementation of the operators

(case-sensitive #t)
(load "datatype.ss")
(load "seq.ss")
(load "cps.ss")

(define-syntax pushPrompt
  (syntax-rules ()
    [(_ p e1 e2 ...)
     ($pushPrompt p (lambda () e1 e2 ...))]))

(define-syntax pushSubCont
  (syntax-rules ()
    [(_ subk e1 e2 ...)
     ($pushSubCont subk (lambda () e1 e2 ...))]))

(define (cps-newPrompt k) (k (string #\p)))

(define cps-$pushPrompt
  (lambda (k p *th)
    (lambda (mk)
      ((*th k0) (PushP p (PushSeg/t k mk))))))

(define (cps-withSubCont k p f)
  (lambda (mk)
    (let-values ([(subk mk) (splitSeq p mk)])
      ((f k0 (PushSeg/t k subk)) mk))))

(define (cps-$pushSubCont k subk *th)
  (lambda (mk)
    ((*th k0) (appendSeq subk (PushSeg/t k mk)))))

(define (PushSeg/t k mk)
  (if (eq? k k0)
      mk
      (PushSeg k mk)))

(define (runCC cps-th)
  ((cps-th k0) (EmptyS)))

(define (k0 v)
  (lambda (mk)
    (underflow mk v)))

(define (underflow mk v)
  (Seq-case mk
    [(EmptyS) v]
    [(PushP _ mk) (underflow mk v)]
    [(PushSeg k mk) ((k v) mk)]))

(define cps-eval
  (let ([eval (current-eval)])
    (lambda (x)
      (runCC (eval (cps (parameterize ([current-eval eval])
                          (expand x))))))))

(define (go)
  (parameterize ([current-eval cps-eval])
    (new-cafe)))
