#lang racket/base
(require racket/include
         (only-in racket/base
                  [define r:define])
         (for-syntax racket/base))

(begin-for-syntax
  (define datum->syntax-object datum->syntax)
  (define syntax-object->datum syntax->datum)
  (define (literal-identifier=? a b)
    (eq? (syntax->datum a) (syntax->datum b)))
  (define (syntax-error . args)
    (apply error args)))

(define-syntax-rule (module exprs body ...)
  (begin body ...))

(define-syntax define
  (syntax-rules ()
    [(_ id) (r:define id (void))]
    [(_ id rhs ...) (r:define id rhs ...)]))

(define-syntax define-record
  (syntax-rules (immutable)
    [(_ name ((immutable field) ...))
     (define-struct name (field ...))]
    [(_ name parent ((immutable field) ...) ())
     (define-struct (name parent) (field ...) #:omit-define-syntaxes)]))

(define-syntax-rule (case-sensitive #t)
  (void))

(define new-cafe void)

(define saved-base-k)

(define (call/cc/base f)
  (with-continuation-mark
   'base-continuation #t
   (call-with-current-continuation
    (lambda (k)
      (set! saved-base-k k)
      (f k)))))

(define (call/cc f)
  (call-with-immediate-continuation-mark
   'base-continuation
   (lambda (base?)
     (if base?
         (f saved-base-k)
         (call-with-current-continuation f)))))

(define (base-continuation? k)
  (and (continuation-mark-set-first (continuation-marks k) 'base-continuation #f)
       (log-error "same")))

(include "tail.ss")
(include "triple.ss")

(define N (string->number (vector-ref (current-command-line-arguments) 0)))
(define runs (string->number (vector-ref (current-command-line-arguments) 1)))

(for/fold ([v #f]) ([i (in-range runs)])
  (time (length (runCC (lambda () (handled-triples-list N N))))))

